=== modified file 'drizzled/util/convert.cc'
--- drizzled/util/convert.cc	2009-04-05 17:35:26 +0000
+++ drizzled/util/convert.cc	2009-11-26 02:50:58 +0000
@@ -19,6 +19,11 @@
  */
 
 #include <drizzled/util/convert.h>
+#include <string>
+#include <iomanip>
+#include <sstream>
+
+using namespace std;
 
 uint64_t drizzled_string_to_hex(char *to, const char *from, uint64_t from_size)
 {
@@ -35,3 +40,38 @@
 
   return from_size * 2;
 }
+
+void bytesToHexdumpFormat(string &to, const unsigned char *from, size_t from_length)
+{
+  static const char hex_map[]= "0123456789abcdef";
+  unsigned int x, y;
+  ostringstream line_number;
+
+  for (x= 0; x < from_length; x+= 16)
+  {
+    line_number << setfill('0') << setw(6);
+    line_number << x;
+    to.append(line_number.str());
+    to.append(": ", 2);
+
+    for (y= 0; y < 16; y++)
+    {
+      if ((x + y) < from_length)
+      {
+        to.push_back(hex_map[(from[x+y]) >> 4]);
+        to.push_back(hex_map[(from[x+y]) & 0xF]);
+        to.push_back(' ');
+      }
+      else
+        to.append("   ");
+    }
+    to.push_back(' ');
+    for (y= 0; y < 16; y++)
+    {
+      if ((x + y) < from_length)
+        to.push_back(isprint(from[x + y]) ? from[x + y] : '.');
+    }
+    to.push_back('\n');
+    line_number.str("");
+  }
+}

=== modified file 'drizzled/util/convert.h'
--- drizzled/util/convert.h	2009-08-30 00:26:17 +0000
+++ drizzled/util/convert.h	2009-11-26 02:50:58 +0000
@@ -42,6 +42,8 @@
   return str;
 }
 
+void bytesToHexdumpFormat(std::string &s, const unsigned char *from, size_t from_length);
+
 extern "C"
 {
   uint64_t drizzled_string_to_hex(char *to, const char *from,

=== added file 'plugin/transaction_log/hexdump_transaction_message.cc'
--- plugin/transaction_log/hexdump_transaction_message.cc	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/hexdump_transaction_message.cc	2009-11-26 02:50:58 +0000
@@ -0,0 +1,180 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  Authors:
+ *
+ *  Jay Pipes <joinfu@sun.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Implements the HEXDUMP_TRANSACTION_MESSAGE(filename, offset) UDF.
+ */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/plugin/function.h>
+#include <drizzled/item/func.h>
+#include <drizzled/function/str/strfunc.h>
+#include <drizzled/error.h>
+#include <drizzled/hash/crc32.h>
+
+#include "transaction_log.h"
+#include "hexdump_transaction_message.h"
+
+#include <drizzled/message/transaction.pb.h>
+#include <drizzled/util/convert.h>
+#include <google/protobuf/io/zero_copy_stream.h>
+#include <google/protobuf/io/zero_copy_stream_impl.h>
+#include <google/protobuf/io/coded_stream.h>
+#include <google/protobuf/text_format.h>
+
+using namespace std;
+using namespace drizzled;
+using namespace google;
+
+/** Defined in transaction_log.cc */
+extern TransactionLog *transaction_log;
+
+plugin::Create_function<HexdumpTransactionMessageFunction> *hexdump_transaction_message_func_factory= NULL;
+
+void HexdumpTransactionMessageFunction::fix_length_and_dec()
+{
+  max_length= 2 * 1024 * 1024; /* 2MB size limit seems ok... */
+  args[0]->collation.set(
+    get_charset_by_csname(args[0]->collation.collation->csname,
+                          MY_CS_BINSORT), DERIVATION_COERCIBLE);
+}
+
+String *HexdumpTransactionMessageFunction::val_str(String *str)
+{
+  assert(fixed == true);
+
+  String *filename_arg= args[0]->val_str(str);
+  off_t offset_arg= static_cast<int64_t>(args[1]->val_int());
+
+  if (filename_arg == NULL || args[1]->null_value == true)
+  {
+    my_error(ER_INVALID_NULL_ARGUMENT, MYF(0), func_name());
+    null_value= true;
+    return NULL;
+  }
+
+  if (transaction_log == NULL)
+  {
+    my_error(ER_INVALID_NULL_ARGUMENT, MYF(0), func_name());
+    null_value= true;
+    return NULL;
+  }
+
+  null_value= false;
+
+  message::Transaction transaction_message;
+
+  /**
+   * @todo Of course, this is not efficient to create a
+   * new input stream every time we call the UDF.  Create
+   * a pool of TransactionLogReader objects that can be 
+   * re-used.
+   */
+  const string &filename= transaction_log->getLogFilename();
+  int log_file= open(filename.c_str(), O_RDONLY);
+  if (log_file == -1)
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, _("Failed to open transaction log file %s.  Got error: %s\n"), 
+                  filename.c_str(), 
+                  strerror(errno));
+    null_value= true;
+    return NULL;
+  }
+
+  (void) lseek(log_file, offset_arg, SEEK_SET);
+
+  protobuf::io::FileInputStream *file_input= new protobuf::io::FileInputStream(log_file);
+  file_input->SetCloseOnDelete(true);
+
+  protobuf::io::CodedInputStream *coded_input= new protobuf::io::CodedInputStream(file_input);
+
+  /* Grab our message type and length */
+  uint32_t message_type;
+  if (! coded_input->ReadLittleEndian32(&message_type))
+  {
+    delete file_input;
+
+    /** @todo Error message for this... */
+    null_value= true;
+    return NULL;
+  }
+
+  uint32_t message_size;
+  if (! coded_input->ReadLittleEndian32(&message_size))
+  {
+    delete file_input;
+
+    /** @todo Error message for this... */
+    null_value= true;
+    return NULL;
+  }
+
+  uint8_t *buffer= (uint8_t *) malloc(message_size);
+
+  bool result= coded_input->ReadRaw(buffer, message_size);
+  if (result == false)
+  {
+    fprintf(stderr, _("Could not read transaction message.\n"));
+    fprintf(stderr, _("GPB ERROR: %s.\n"), strerror(errno));
+    fprintf(stderr, _("Raw buffer read: %s.\n"), buffer);
+  }
+
+  /*
+   * Convert raw bytes to a hex representation and store back into
+   * the return String.
+   */
+  string hexdump;
+  hexdump.reserve(message_size * 4);
+  bytesToHexdumpFormat(hexdump, reinterpret_cast<const unsigned char *>(buffer), message_size);
+
+  /* 
+   * Check that the transaction message is actually not corrupt before
+   * printing out the raw byte representation of the transaction message...
+   */
+  result= transaction_message.ParseFromArray(buffer, static_cast<int32_t>(message_size));
+  if (result == false)
+  {
+    fprintf(stderr, _("Unable to parse transaction. Got error: %s.\n"), transaction_message.InitializationErrorString().c_str());
+    if (buffer != NULL)
+      fprintf(stderr, _("BUFFER: %s\n"), buffer);
+  }
+
+  if (str->alloc(message_size * 4)) /* Hex representation is ~4 times number of bytes */
+  {
+    null_value= true;
+    return NULL;
+  }
+
+  strncpy(str->ptr(), hexdump.c_str(), hexdump.length());
+  str->length(hexdump.length());
+
+  free(buffer);
+
+  delete coded_input;
+  delete file_input;
+
+  return str;
+}

=== added file 'plugin/transaction_log/hexdump_transaction_message.h'
--- plugin/transaction_log/hexdump_transaction_message.h	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/hexdump_transaction_message.h	2009-11-26 02:50:58 +0000
@@ -0,0 +1,65 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  Authors:
+ *
+ *  Jay Pipes <joinfu@sun.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the HEXDUMP_TRANSACTION_MESSAGE() UDF
+ *
+ * @details
+ *
+ * HEXDUMP_TRANSACTION_MESSAGE(filename, offset);
+ *
+ * prints a text representation of the raw bytes which make up
+ * the transaction message in the logfile at the supplied offset
+ */
+
+#ifndef PLUGIN_TRANSACTION_LOG_HEXDUMP_TRANSACTION_MESSAGE_H
+#define PLUGIN_TRANSACTION_LOG_HEXDUMP_TRANSACTION_MESSAGE_H
+
+#include <drizzled/server_includes.h>
+#include <drizzled/item/func.h>
+#include <drizzled/function/str/strfunc.h>
+
+class HexdumpTransactionMessageFunction : public Item_str_func
+{
+public:
+  HexdumpTransactionMessageFunction() : Item_str_func() {}
+  String *val_str(String*);
+
+  void fix_length_and_dec();
+
+  const char *func_name() const
+  {
+    return "hexdump_transaction_message";
+  }
+
+  bool check_argument_count(int n)
+  {
+    return (n == 2);
+  }
+};
+
+#endif /* PLUGIN_TRANSACTION_LOG_HEXDUMP_TRANSACTION_MESSAGE_H */
+

=== modified file 'plugin/transaction_log/info_schema.cc'
--- plugin/transaction_log/info_schema.cc	2009-11-16 22:00:02 +0000
+++ plugin/transaction_log/info_schema.cc	2009-11-26 02:50:58 +0000
@@ -54,7 +54,7 @@
  * , START_TIMESTAMP BIGINT NOT NULL
  * , END_TIMESTAMP BIGINT NOT NULL
  * , NUM_STATEMENTS INT NOT NULL
- * , CHECKSUM INT NOT NULL
+ * , CHECKSUM BIGINT NOT NULL // had to be BIGINT b/c we have no unsigned int.
  * );
  */
 

=== modified file 'plugin/transaction_log/plugin.ini'
--- plugin/transaction_log/plugin.ini	2009-11-20 18:08:46 +0000
+++ plugin/transaction_log/plugin.ini	2009-11-26 02:50:58 +0000
@@ -3,8 +3,9 @@
 title=Transaction Log
 description=Log of Transaction Messages
 load_by_default=yes
-sources= background_worker.cc info_schema.cc print_transaction_message.cc transaction_log.cc transaction_log_entry.cc transaction_log_index.cc transaction_log_reader.cc 
-headers= background_worker.h info_schema.h print_transaction_message.h transaction_log.h transaction_log_entry.h transaction_log_index.h transaction_log_reader.h
+sources= background_worker.cc info_schema.cc hexdump_transaction_message.cc print_transaction_message.cc transaction_log.cc transaction_log_entry.cc transaction_log_index.cc transaction_log_reader.cc 
+headers= background_worker.h info_schema.h hexdump_transaction_message.h print_transaction_message.h transaction_log.h transaction_log_entry.h transaction_log_index.h transaction_log_reader.h
 testsuite=transaction_log
-ldflags=${top_builddir}/drizzled/hash/libhash.la
+ldflags=${top_builddir}/drizzled/hash/libhash.la 
+libadd=$(LIBZ)
 static=yes

=== modified file 'plugin/transaction_log/print_transaction_message.cc'
--- plugin/transaction_log/print_transaction_message.cc	2009-11-16 22:00:02 +0000
+++ plugin/transaction_log/print_transaction_message.cc	2009-11-26 02:50:58 +0000
@@ -25,7 +25,7 @@
 /**
  * @file
  *
- * Implements the PRINT_TRANSACTION_MESSAGE() UDF.
+ * Implements the PRINT_TRANSACTION_MESSAGE(filename, offset) UDF.
  */
 
 #include <drizzled/server_includes.h>
@@ -50,7 +50,7 @@
 /** Defined in transaction_log.cc */
 extern TransactionLog *transaction_log;
 
-plugin::Create_function<PrintTransactionMessageFunction> *print_transaction_message_func= NULL;
+plugin::Create_function<PrintTransactionMessageFunction> *print_transaction_message_func_factory= NULL;
 
 void PrintTransactionMessageFunction::fix_length_and_dec()
 {
@@ -163,6 +163,7 @@
 
   strncpy(str->ptr(), transaction_text.c_str(), transaction_text.length());
 
+  delete coded_input;
   delete file_input;
 
   return str;

=== modified file 'plugin/transaction_log/transaction_log.cc'
--- plugin/transaction_log/transaction_log.cc	2009-11-20 20:22:37 +0000
+++ plugin/transaction_log/transaction_log.cc	2009-11-26 02:50:58 +0000
@@ -75,6 +75,7 @@
 #include "transaction_log_index.h"
 #include "info_schema.h"
 #include "print_transaction_message.h"
+#include "hexdump_transaction_message.h"
 #include "background_worker.h"
 
 #include <unistd.h>
@@ -124,7 +125,8 @@
 extern TransactionLogIndex *transaction_log_index;
 
 /** Defined in print_transaction_message.cc */
-extern plugin::Create_function<PrintTransactionMessageFunction> *print_transaction_message_func;
+extern plugin::Create_function<PrintTransactionMessageFunction> *print_transaction_message_func_factory;
+extern plugin::Create_function<HexdumpTransactionMessageFunction> *hexdump_transaction_message_func_factory;
 
 TransactionLog::TransactionLog(string name_arg,
                                const string &in_log_file_path,
@@ -407,9 +409,13 @@
     registry.add(transaction_log_transactions_view);
 
     /* Setup the module's UDFs */
-    print_transaction_message_func=
+    print_transaction_message_func_factory=
       new plugin::Create_function<PrintTransactionMessageFunction>("print_transaction_message");
-    registry.add(print_transaction_message_func);
+    registry.add(print_transaction_message_func_factory);
+
+    hexdump_transaction_message_func_factory=
+      new plugin::Create_function<HexdumpTransactionMessageFunction>("hexdump_transaction_message");
+    registry.add(hexdump_transaction_message_func_factory);
 
     /* Create and initialize the transaction log index */
     transaction_log_index= new (nothrow) TransactionLogIndex(*transaction_log);
@@ -459,8 +465,10 @@
     cleanupViews();
 
     /* Cleanup module UDFs */
-    registry.remove(print_transaction_message_func);
-    delete print_transaction_message_func;
+    registry.remove(print_transaction_message_func_factory);
+    delete print_transaction_message_func_factory;
+    registry.remove(hexdump_transaction_message_func_factory);
+    delete hexdump_transaction_message_func_factory;
   }
 
   return 0;

=== modified file 'plugin/transaction_log/transaction_log.h'
--- plugin/transaction_log/transaction_log.h	2009-11-16 22:00:02 +0000
+++ plugin/transaction_log/transaction_log.h	2009-11-26 02:50:58 +0000
@@ -171,7 +171,7 @@
   void clearError();
 
   int log_file; ///< Handle for our log file
-  enum Status state; ///< The state the log is in
+  Status state; ///< The state the log is in
   drizzled::atomic<bool> do_checksum; ///< Do a CRC32 checksum when writing Transaction message to log?
   const std::string log_file_path; ///< Full path to the log file
   std::string log_file_name; ///< Name of the log file

=== modified file 'tests/suite/transaction_log/r/information_schema.result'
--- tests/suite/transaction_log/r/information_schema.result	2009-11-20 20:22:37 +0000
+++ tests/suite/transaction_log/r/information_schema.result	2009-11-26 02:50:58 +0000
@@ -10,14 +10,14 @@
 transaction.log	454	4	4	0	4	X	X
 SELECT * FROM INFORMATION_SCHEMA.TRANSACTION_LOG_ENTRIES;
 ENTRY_OFFSET	ENTRY_TYPE	ENTRY_LENGTH
-0	TRANSACTION	85
-85	TRANSACTION	127
-212	TRANSACTION	121
-333	TRANSACTION	121
+X	TRANSACTION	85
+X	TRANSACTION	127
+X	TRANSACTION	121
+X	TRANSACTION	121
 SELECT * FROM INFORMATION_SCHEMA.TRANSACTION_LOG_TRANSACTIONS;
 ENTRY_OFFSET	TRANSACTION_ID	SERVER_ID	START_TIMESTAMP	END_TIMESTAMP	NUM_STATEMENTS	CHECKSUM
-0	0	1	X	X	1	0
-85	0	1	X	X	1	0
-212	3	1	X	X	1	0
-333	4	1	X	X	1	0
+X	0	1	X	X	1	0
+X	0	1	X	X	1	0
+X	3	1	X	X	1	0
+X	4	1	X	X	1	0
 SET GLOBAL transaction_log_truncate_debug= true;

=== modified file 'tests/suite/transaction_log/t/information_schema.test'
--- tests/suite/transaction_log/t/information_schema.test	2009-11-20 20:22:37 +0000
+++ tests/suite/transaction_log/t/information_schema.test	2009-11-26 02:50:58 +0000
@@ -12,8 +12,9 @@
 
 --replace_column 7 X 8 X
 SELECT * FROM INFORMATION_SCHEMA.TRANSACTION_LOG;
+--replace_column 1 X
 SELECT * FROM INFORMATION_SCHEMA.TRANSACTION_LOG_ENTRIES;
---replace_column 4 X 5 X
+--replace_column 1 X 4 X 5 X
 SELECT * FROM INFORMATION_SCHEMA.TRANSACTION_LOG_TRANSACTIONS;
 
 # Truncate the log file to reset for the next test

